package org.snap.jounin;

import org.planet.p2pcm.*;

import dermi.*;
import dermi.core.*;
import dermi.exception.*;
import dermi.annotation.*;
import dermi.util.pastry.*;

import java.util.*;

@DCMImplementation
public class ParticipantImpl extends Component implements Participant, StatefulReplica {
	private static final long serialVersionUID = 345702345;
	
	private String _name;
	private Id _pass;
	private ArrayList<String> _portals;
	private Hashtable<String,ParticipantProfile> _profiles;
	
	// Dermi constructor: REQUIRED
	public ParticipantImpl() {
	}

	// Dermi constructor: REQUIRED
	public ParticipantImpl(Properties env) throws RemoteException {
		super(env);

		// Load replica state: as in any other stateful Dermi remote object
		super.loadReplicaState();
	}

	// Constructor with arguments. It creates a new Participant and stores it. 
	public ParticipantImpl(Properties env, String username, String pass) throws RemoteException {
		super(env);
		_name = username;
		_pass = HashGenerator.generateHash(pass);
		_portals = new ArrayList<String>();
		_profiles = new Hashtable<String,ParticipantProfile>();

		// Load replica state: as in any other stateful Dermi remote object
		super.loadReplicaState();
	}
	
	public ParticipantProfile getPortletProfile(String portlet, String portal) throws RemoteException {
		ParticipantProfile profile = null;
		profile = _profiles.get(portlet+portal);
		if(profile == null){
			profile = new ParticipantProfile(portlet+portal);
			_profiles.put(portlet+portal, profile);
		}
		return profile;
	}
	
	public void delete() throws RemoteException {
		try{
			this.passivate();
		}catch(PassivationException e){
			throw new RemoteException("Failed while passivating component portal");
		}
		this.deletePersistentState();
	}
	
	public String getName() throws RemoteException {
		return _name;
	}
	
	public Id getPassword() throws RemoteException {
		return _pass;
	}

	public Collection getPortals() throws RemoteException {
		return _portals;
	}

	public void addPortal(String portal) throws RemoteException {
		if(!_portals.contains(portal))
			_portals.add(portal);
	}
	
	public void delPortal(String portal) throws RemoteException {
		_portals.remove(portal);
	}

	/**
	 * The queryInterface method must return a suitable component implementation
	 * for the specified interface
	 * 
	 * @param interf
	 *            String The desired interface implementation
	 * @return ComponentInterface The desired implementation instance
	 * @throws InterfaceNotFoundException
	 *             If the component does not provide an implementation for such
	 *             specified interface
	 */
	public ComponentInterface queryInterface(String interf) throws InterfaceNotFoundException {
		try {
			if (interf.equals("org.planet.p2pcm.ComponentInterface") || interf.equals("org.snap.jounin.Participant")) {
				return (ComponentInterface) this.copy();
			} else {
				throw new InterfaceNotFoundException("Interface " + interf + " not implemented.");
			}
		} catch (RemoteException re) {
			throw new InterfaceNotFoundException("Unable to query for interface " + interf + ": " + re.getMessage());
		}
	}

	/**
	 * Method for loading state into this object
	 */
	public void loadRemoteObjectState() throws RemoteException {
		ArrayList<Object> temp = (ArrayList<Object>) super.loadState();
		_name = (String) temp.get(0);
		_pass = (Id) temp.get(1);
		_portals = (ArrayList)temp.get(2);
		_profiles = (Hashtable) temp.get(3);
		System.out.println("[Participant] - state loaded from network.");
	}

	/**
	 * Method for obtaining this object's state
	 */
	public java.io.Serializable getRemoteObjectState() {
		ArrayList<Object> temp = new ArrayList<Object>();
		temp.add(0, _name);
		temp.add(1, _pass);
		temp.add(2, _portals);
		temp.add(3, _profiles);
		return (java.io.Serializable) temp;
	}

}
